﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;

namespace Gbjam5MksGame
{
    public enum GameMode { None, Title, Help, Configure, GameWin, GameOver, Ingame };
    public enum UpdateLayer { First, Last };
    public enum DisplayLayer
    {
        VisualBackground,
        MapBackground,
        MapLayer0Tiles, MapLayer0Shadow, MapLayer0Sprites,
        MapLayer1Tiles, MapLayer1Shadow, MapLayer1Sprites,
        MapLayer2Tiles, MapLayer2Shadow, MapLayer2Sprites,
        MapLayer3Tiles, MapLayer3Shadow, MapLayer3Sprites,
        MapLayer4Tiles, MapLayer4Shadow, MapLayer4Sprites,
        MapForeground,
        UI
    };

    public class MyGame : Game
    {
        public GraphicsDeviceManager Graphics;
        public SpriteBatch SpriteBatch;

        public Texture2D TitleScreenTexture;
        public Texture2D HelpScreenTexture;
        public Texture2D ConfigureScreenTexture;
        public Texture2D GameWinScreenTexture;
        public Texture2D GameOverScreenTexture;
        public Texture2D IngameScreenTexture;
        public Texture2D BlankScreenTexture;

        public Texture2D TilesTexture;

        public RenderTarget2D BackgroundRenderTarget;

        public SoundEffect MenuNavSound;
        public SoundEffect MenuSelectSound;
        public SoundEffect PlayerFireShotSound;
        public SoundEffect EnemyFireShotSound;
        public SoundEffect EnemyFireMissileSound;
        public SoundEffect SmallHitSound;
        public SoundEffect MediumHitSound;
        public SoundEffect LargeHitSound;
        public SoundEffect StageClearSound;
        public SoundEffect GameWinSound;
        public SoundEffect GameLossSound;

        BackgroundPlateGameVisual backgroundPlateVisual;
        MapManagerGameComponent mapManagerComponent;
        UiManagerGameComponent uiManagerComponent;

        public GameMode CurrentGameMode = GameMode.Title;
        public GameMode PreviousGameMode = GameMode.None;

        public bool SfxOn = true;
        public bool MusicOn = true;

        bool quitGame = false;

        public int Hiscore = 0;
        SpriteGameVisual playerHiscoreTextSprite;
        SpriteGameVisual[] playerHiscoreSprites;

        public bool MoveUpPressed = false;
        public bool MoveDownPressed = false;
        public bool MoveLeftPressed = false;
        public bool MoveRightPressed = false;
        public bool FirePressed = false;
        public bool PausePressed = false;
        public bool BackPressed = false;

        public MyGame()
        {
            Graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            // use command line to set window size: EXE w=960 h=864 f=[yes/no]
            int chosenWidth = 960;          // 640; 160; 960;
            int chosenHeight = 864;         // 576; 144;  864;
            bool chosenFullscreen = false;

            try
            {

                string[] args = Environment.GetCommandLineArgs();
                if (args.Length > 1)
                {
                    for (int index = 1; index < args.Length; index++)
                    {
                        string keyValue = args[index];
                        if (keyValue.Contains("="))
                        {
                            char[] splitters = new char[] { '=' };
                            string[] values = keyValue.Split(splitters);

                            if (values[0].Equals("w"))
                            {
                                int setWidth = 0;
                                int.TryParse(values[1], out setWidth);
                                if (setWidth != 0)
                                {
                                    chosenWidth = setWidth;
                                }
                            }
                            else if (values[0].Equals("h"))
                            {
                                int setHeight = 0;
                                int.TryParse(values[1], out setHeight);
                                if (setHeight != 0)
                                {
                                    chosenHeight = setHeight;
                                }
                            }
                            else if (values[0].Equals("f"))
                            {
                                string setFullscreen = values[1];
                                if (setFullscreen.ToLower().Equals("yes"))
                                {
                                    chosenFullscreen = true;
                                }
                                else 
                                {
                                    chosenFullscreen = false;
                                }
                            }
                        }
                    }
                }
            }
            catch
            {
            }
            finally
            {
                // v1.0 - Build 09.10.2016
                this.Window.Title = "Assault On Metaluna 6 - v1.0"; 

                Graphics.IsFullScreen = chosenFullscreen;
                Graphics.PreferredBackBufferWidth = chosenWidth;
                Graphics.PreferredBackBufferHeight = chosenHeight;
                Graphics.ApplyChanges();
            }

            base.Initialize();
        }

        protected override void LoadContent()
        {
            SpriteBatch = new SpriteBatch(GraphicsDevice);

            TitleScreenTexture = this.Content.Load<Texture2D>("Images/title");
            HelpScreenTexture = this.Content.Load<Texture2D>("Images/help");
            ConfigureScreenTexture = this.Content.Load<Texture2D>("Images/configure");
            GameWinScreenTexture = this.Content.Load<Texture2D>("Images/game_won");
            GameOverScreenTexture = this.Content.Load<Texture2D>("Images/game_over");
            IngameScreenTexture = this.Content.Load<Texture2D>("Images/ingame");
            BlankScreenTexture = this.Content.Load<Texture2D>("Images/blank");
            TilesTexture = this.Content.Load<Texture2D>("Images/tiles1");

            // ---
            // private SoundEffect effect;
            // effect = Content.Load<SoundEffect>("SoundFX/ExtraLife");
            // effect.Play();
            // float volume = 1.0f;
            // float pitch = 0.0f;
            // float pan = 0.0f;
            // effect.Play(volume, pitch, pan);
            //SoundEffect testSound = Content.Load<SoundEffect>("SoundFX/mpthreetest");
            //testSound.Play();
            //
            MenuNavSound = Content.Load<SoundEffect>("SoundFX/menu_nav");
            MenuSelectSound = Content.Load<SoundEffect>("SoundFX/menu_select1");
            PlayerFireShotSound = Content.Load<SoundEffect>("SoundFX/shoot6");
            EnemyFireShotSound = Content.Load<SoundEffect>("SoundFX/shoot2");
            EnemyFireMissileSound = Content.Load<SoundEffect>("SoundFX/missileShoot1");
            SmallHitSound = Content.Load<SoundEffect>("SoundFX/hit1");
            MediumHitSound = Content.Load<SoundEffect>("SoundFX/hit2");
            LargeHitSound = Content.Load<SoundEffect>("SoundFX/hit3");
            StageClearSound = Content.Load<SoundEffect>("SoundFX/stageClear3");
            GameWinSound = Content.Load<SoundEffect>("SoundFX/stageClear3");
            GameLossSound = Content.Load<SoundEffect>("SoundFX/stageClear1");

            // ---
            BackgroundRenderTarget = new RenderTarget2D(GraphicsDevice, 160, 144, false, GraphicsDevice.PresentationParameters.BackBufferFormat, DepthFormat.Depth24);

            // ---
            backgroundPlateVisual = new BackgroundPlateGameVisual(this);
            backgroundPlateVisual.Enabled = false;
            backgroundPlateVisual.Visible = true;
            this.Components.Add(backgroundPlateVisual);

            mapManagerComponent = new MapManagerGameComponent(this);
            mapManagerComponent.Enabled = true;
            this.Components.Add(mapManagerComponent);

            uiManagerComponent = new UiManagerGameComponent(this);
            uiManagerComponent.Enabled = true;
            this.Components.Add(uiManagerComponent);

            // ---
            Hiscore = 0;
            string hiscoreText = string.Empty;
            try
            {
                System.IO.StreamReader file = new System.IO.StreamReader(@"hiscore.txt");
                hiscoreText = file.ReadLine();
                file.Close();
            }
            catch
            {
            }
            finally
            {
                int.TryParse(hiscoreText, out Hiscore);
            }

            playerHiscoreTextSprite = new SpriteGameVisual(this);
            playerHiscoreTextSprite.DrawOrder = (int)DisplayLayer.UI;
            playerHiscoreTextSprite.Enabled = false;
            playerHiscoreTextSprite.UpdateOrder = (int)UpdateLayer.Last;
            playerHiscoreTextSprite.SourceImage = TilesTexture;
            playerHiscoreTextSprite.SourceRectangle = new Rectangle(32, 288, 57, 9);
            playerHiscoreTextSprite.TargetPosition = new Vector2(22.0f, 4.0f);
            playerHiscoreTextSprite.TargetSize = new Vector2(57.0f, 9.0f);
            playerHiscoreTextSprite.Visible = true;
            Components.Add(playerHiscoreTextSprite);

            playerHiscoreSprites = new SpriteGameVisual[7];
            for (int scoreIndex = 0; scoreIndex <= 6; scoreIndex++)
            {
                playerHiscoreSprites[scoreIndex] = new SpriteGameVisual(this);
                playerHiscoreSprites[scoreIndex].DrawOrder = (int)DisplayLayer.UI;
                playerHiscoreSprites[scoreIndex].Enabled = false;
                playerHiscoreSprites[scoreIndex].UpdateOrder = (int)UpdateLayer.Last;
                playerHiscoreSprites[scoreIndex].SourceImage = TilesTexture;
                playerHiscoreSprites[scoreIndex].SourceRectangle = new Rectangle(48, 240, 8, 8);
                playerHiscoreSprites[scoreIndex].TargetPosition = new Vector2(82.0f + (scoreIndex * 8.0f), 4.0f);
                playerHiscoreSprites[scoreIndex].TargetSize = new Vector2(8.0f, 8.0f);
                playerHiscoreSprites[scoreIndex].Visible = true;
                Components.Add(playerHiscoreSprites[scoreIndex]);
            }
            UpdatePlayerHiscoreDisplay();
        }

        protected override void UnloadContent()
        {
            System.IO.File.WriteAllText(@"hiscore.txt", Hiscore.ToString());
        }

        protected override void Update(GameTime gameTime)
        {
            GamePadState gamePadState = GamePad.GetState(PlayerIndex.One);
            KeyboardState keyboardState = Keyboard.GetState();

            MoveUpPressed = keyboardState.IsKeyDown(Keys.W) || gamePadState.DPad.Up == ButtonState.Pressed;
            MoveDownPressed = keyboardState.IsKeyDown(Keys.S) || gamePadState.DPad.Down == ButtonState.Pressed;
            MoveLeftPressed = keyboardState.IsKeyDown(Keys.A) || gamePadState.DPad.Left == ButtonState.Pressed;
            MoveRightPressed = keyboardState.IsKeyDown(Keys.D) || gamePadState.DPad.Right == ButtonState.Pressed;
            FirePressed = keyboardState.IsKeyDown(Keys.K) || gamePadState.Buttons.A == ButtonState.Pressed;
            PausePressed = keyboardState.IsKeyDown(Keys.P) || gamePadState.Buttons.B == ButtonState.Pressed;
            BackPressed = keyboardState.IsKeyDown(Keys.Escape) || gamePadState.Buttons.Back == ButtonState.Pressed;

            if (CurrentGameMode == GameMode.Title)
            {
                if (BackPressed || quitGame)
                {
                    Exit();
                }

                #region Test Code
                //if (state.IsKeyDown(Keys.F1))
                //{
                //    SwitchToGameMode(GameMode.Title);
                //}
                //else if (state.IsKeyDown(Keys.F2))
                //{
                //    SwitchToGameMode(GameMode.Configure);
                //}
                //else if (state.IsKeyDown(Keys.F3))
                //{
                //    SwitchToGameMode(GameMode.GameOver);
                //}
                //else if (state.IsKeyDown(Keys.F4))
                //{
                //    SwitchToGameMode(GameMode.GameWin);
                //}
                //else if (state.IsKeyDown(Keys.F5))
                //{
                //    SwitchToGameMode(GameMode.Help);
                //}
                //else if (state.IsKeyDown(Keys.F6))
                //{
                //    SwitchToGameMode(GameMode.Ingame);
                //}

                //if (state.IsKeyDown(Keys.V) && backgroundPlateVisual.DoBlankScreen == false)
                //{
                //    backgroundPlateVisual.DoBlankScreen = true;
                //}
                //else if (state.IsKeyDown(Keys.B) && backgroundPlateVisual.DoBlankScreen == true)
                //{
                //    backgroundPlateVisual.DoBlankScreen = false;
                //}

                #endregion
            }

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            // Window ratio calculations
            float sourceWidth = 160.0f;
            float sourceHeight = 144.0f;

            float clientWidth = (float)GraphicsDevice.PresentationParameters.BackBufferWidth;
            float clientHeight = (float)GraphicsDevice.PresentationParameters.BackBufferHeight;

            float finalRequiredWidth = clientWidth;
            float finalRequiredHeight = clientHeight;

            int xOffset = 0;
            int yOffset = 0;

            if (clientWidth >= clientHeight)
            {
                float hRatio = clientHeight / sourceHeight;
                finalRequiredWidth = hRatio * sourceWidth;

                xOffset = (int)((clientWidth - finalRequiredWidth) / 2.0);
            }
            else
            {
                float vRatio = clientWidth / sourceWidth;
                finalRequiredHeight = vRatio * sourceHeight;

                yOffset = (int)((clientHeight - finalRequiredHeight) / 2.0);
            }

            // Drawing 
            GraphicsDevice.SetRenderTarget(null);

            GraphicsDevice.Clear(Color.Black);

            SpriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);

            SpriteBatch.Draw(BackgroundRenderTarget, new Rectangle(xOffset, yOffset, (int)finalRequiredWidth, (int)finalRequiredHeight), Color.White);

            SpriteBatch.End();

            base.Draw(gameTime);
        }

        public void SwitchToGameMode(GameMode newGameMode)
        {
            if (newGameMode == CurrentGameMode)
            {
                return;
            }

            PreviousGameMode = CurrentGameMode;
            CurrentGameMode = newGameMode;
        }

        public void SwitchToTitleScreen()
        {
            SwitchToGameMode(GameMode.Title);

            ShowHiscoreSprites(true);
        }

        public void StartNewGame()
        {
            ShowHiscoreSprites(false);

            mapManagerComponent.PrepareNewGame();
            SwitchToGameMode(GameMode.Ingame);
        }

        public void SwitchToHelpScreen()
        {
            ShowHiscoreSprites(false);

            SwitchToGameMode(GameMode.Help);
        }

        public void SwitchToConfigureScreen()
        {
            ShowHiscoreSprites(false);

            SwitchToGameMode(GameMode.Configure);
        }

        public void SwitchToGameWinScreen()
        {
            ShowHiscoreSprites(false);

            if (SfxOn)
            {
                GameWinSound.Play();
            }

            SwitchToGameMode(GameMode.GameWin);
        }

        public void SwitchToGameOverScreen()
        {
            ShowHiscoreSprites(false);

            if (SfxOn)
            {
                GameLossSound.Play();
            }

            SwitchToGameMode(GameMode.GameOver);
        }

        public void QuitApplication()
        {
            quitGame = true;
        }

        void ShowHiscoreSprites(bool flag)
        {
            if (playerHiscoreTextSprite.Visible != flag)
            {
                playerHiscoreTextSprite.Visible = flag;
            }

            for (int scoreIndex = 0; scoreIndex <= 6; scoreIndex++)
            {
                if (playerHiscoreSprites[scoreIndex].Visible != flag)
                {
                    playerHiscoreSprites[scoreIndex].Visible = flag;
                }
            }
        }

        void UpdatePlayerHiscoreDisplay()
        {
            int currentRemain = Hiscore;

            int digit0 = currentRemain / 1000000;
            currentRemain -= digit0 * 1000000;

            int digit1 = currentRemain / 100000;
            currentRemain -= digit1 * 100000;

            int digit2 = currentRemain / 10000;
            currentRemain -= digit2 * 10000;

            int digit3 = currentRemain / 1000;
            currentRemain -= digit3 * 1000;

            int digit4 = currentRemain / 100;
            currentRemain -= digit4 * 100;

            int digit5 = currentRemain / 10;
            currentRemain -= digit5 * 10;

            int digit6 = currentRemain;

            playerHiscoreSprites[0].SourceRectangle.X = 48 + (digit0 * 8);
            playerHiscoreSprites[1].SourceRectangle.X = 48 + (digit1 * 8);
            playerHiscoreSprites[2].SourceRectangle.X = 48 + (digit2 * 8);
            playerHiscoreSprites[3].SourceRectangle.X = 48 + (digit3 * 8);
            playerHiscoreSprites[4].SourceRectangle.X = 48 + (digit4 * 8);
            playerHiscoreSprites[5].SourceRectangle.X = 48 + (digit5 * 8);
            playerHiscoreSprites[6].SourceRectangle.X = 48 + (digit6 * 8);
        }
    }
}
